using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace VamTastic
{
    public class Spencer_Estate : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Hazmhox"; // edited by VamTastic 
        public string pluginName = "Spencer Mansion Doors";
        public string pluginVersion = "2";
        public string pluginDate = "23/02" + "/2022";
        public string pluginDescription = @"Used to open and close the doors";
        #endregion

        #region Vars    



        public GameObject player;
        public GameObject KS_Spencer_Mansion;

        protected JSONStorableFloat ScrubA;
        protected JSONStorableFloat ScrubB;
        protected JSONStorableFloat ScrubC;
        protected JSONStorableFloat ScrubD;
        protected JSONStorableFloat ScrubE;
        protected JSONStorableFloat ScrubF;	
		
	    protected JSONStorableFloat yRotation;
        protected JSONStorableFloat yRotation2;
        protected JSONStorableFloat yRotation3;
        protected JSONStorableFloat yRotation4;
        protected JSONStorableFloat yRotation5;
        protected JSONStorableFloat yRotation6;		


        private float previousValue1;
        private float previousValue2;
		private float previousValue3;
        private float previousValue4;
        private float previousValue5;
        private float previousValue6;		

        private float previousZValue;
        private float currentZValue;

	    public Transform ObjectRoot;
        public Transform door1;
        public Transform door2;
		public Transform door3;
		public Transform door4;
		public Transform door5;
		public Transform door6;		
		
        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute

        #endregion

        public override void Init()
        {
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
				{
	                SuperController.LogError("Spencer_Mansion - Please add this script on a Custom Unity Asset ( script disabled).");
  	                return;
  		        }


                //**************  ScrubA
                ScrubA = new JSONStorableFloat("Bedroom Door Hall", 0f, OnSliderChanged, 0f, 1f);
                RegisterFloat(ScrubA);
                CreateSlider(ScrubA, false);
                          

                //**************  ScrubB
                ScrubB = new JSONStorableFloat("Bedroom Door", 0f, OnSliderChanged2, 0f, 1f);				
                RegisterFloat(ScrubB);
                CreateSlider(ScrubB, false);
				
				
                //**************  ScrubC
                ScrubC = new JSONStorableFloat("Dining Door 1", 0f, OnSliderChanged3, 0f, 1f);
                RegisterFloat(ScrubC);
                CreateSlider(ScrubC, false);
                          

                //**************  ScrubD
                ScrubD = new JSONStorableFloat("Dining Door 2", 0f, OnSliderChanged4, 0f, 1f);				
                RegisterFloat(ScrubD);
                CreateSlider(ScrubD, false);				
                          

                //**************  ScrubE
                ScrubE = new JSONStorableFloat("Front Door 1", 0f, OnSliderChanged5, 0f, 1f);				
                RegisterFloat(ScrubE);
                CreateSlider(ScrubE, false);				

                //**************  ScrubF
                ScrubF = new JSONStorableFloat("Front Door 2", 0f, OnSliderChanged6, 0f, 1f);				
                RegisterFloat(ScrubF);
                CreateSlider(ScrubF, false);				

              // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>About</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Credits to Hazmhox.\n\n" +					
                    "The doors can be triggered using Acidbubbles Timeline to open and close.\n\n" +
                    "if you have a door open and save the scene the position of a door is also saved.\n\n" +
                    "</color>" );
               UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
			// I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
			// In reality, that's even cleaner to do it that way
			if( initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading ) {
                initLoops++;
                InitEditor();
            }
			
			if( initLoops == maxInitLoops ) {
                SuperController.LogError("Spencer_Estate : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }
        }

        private bool InitEditor() {
            if( editorInitialized == true ) return true;
        
            ObjectRoot = getObjectRoot( containingAtom.reParentObject, "Spencer_Mansion" );
            if( ObjectRoot == null ) {
                return false;
            }
			
            door1 = getChildRoot(ObjectRoot, "Bedroom_Door1");
            door2 = getChildRoot(ObjectRoot, "Bedroom_Door2");
			door3 = getChildRoot(ObjectRoot, "Dining_Door1");
            door4 = getChildRoot(ObjectRoot, "Dining_Door2");
            door5 = getChildRoot(ObjectRoot, "Door1");
            door6 = getChildRoot(ObjectRoot, "Door2");						
			 
            // I found my transforms, now the rest
            if( door1 && door2 && door3 && door4 && door5 && door6 ) {
				editorInitialized = true;
				
				// Everything is initialized, triggering the callbacks to init the door rotation saved in the scene file
				// I'm just quickly fixing the script for VamTastic, I'm not gonna refactor everything, so in reality,
				// the float val param of the callbacks is useless
				OnSliderChanged( 0f );
				OnSliderChanged2( 0f );
				OnSliderChanged3( 0f );
				OnSliderChanged4( 0f );
				OnSliderChanged5( 0f );
				OnSliderChanged6( 0f );
			
				return true;
            } else {
                SuperController.LogError("Spencer_Mansion : your CUA is missing mandatory components.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }
        
            return false;
        }

		// Recursive function searching for a transform containing the string set in objectNameSearchString
		// It is used the first time for the initialization of the script
		// This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
		private Transform getObjectRoot( Transform parent, string rootName ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(rootName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child,rootName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		private Transform getChildRoot( Transform parent, string searchName ) {
			foreach( Transform child in parent )
			{
				SuperController.LogMessage(">>" + child.name);
				if( child.name == searchName ) {
					
					return child;
				} else {
					Transform childSearch = getChildRoot(child,searchName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}

		// bedroom Door 1
        void OnSliderChanged( float rotation )
        {
            
            {
				if( door1 == null ) return;
				
                var Slider1 = ScrubA.val;
                var a = Slider1 - previousValue1;
				door1.transform.Rotate(Vector3.up * a * 78);
				previousValue1 = ScrubA.val;
            }

        }

		// Bedroom Door 2
        void OnSliderChanged2( float rotation )
        {

            {
				if( door2 == null ) return;
				
                var Slider2 = ScrubB.val;
                var b = Slider2 - previousValue2;
                door2.transform.Rotate(Vector3.up * b * 86);
                previousValue2 = ScrubB.val;
            }

        }	
		
		// Dining Door L
        void OnSliderChanged3( float rotation )
        {

            {
				if( door3 == null ) return;
				
                var Slider3 = ScrubC.val;
                var c = Slider3 - previousValue3;
                door3.transform.Rotate(Vector3.up * c * -90);
                previousValue3 = ScrubC.val;
            }

        }
		
		// Dining Door R
        void OnSliderChanged4( float rotation )
        {

            {
				if( door4 == null ) return;
				
                var Slider4 = ScrubD.val;
                var d = Slider4 - previousValue4;
                door4.transform.Rotate(Vector3.up * d * 90);
                previousValue4 = ScrubD.val;
            }

        }
		
		// Front Door 1
        void OnSliderChanged5( float rotation )
        {

            {
				if( door5 == null ) return;
				
                var Slider5 = ScrubE.val;
                var e = Slider5 - previousValue5;
                door5.transform.Rotate(Vector3.up * e * 90);
                previousValue5 = ScrubE.val;
            }

        }

		// Front Door 2
        void OnSliderChanged6( float rotation )
        {

            {
				if( door6 == null ) return;
				
                var Slider6 = ScrubF.val;
                var f = Slider6 - previousValue6;
                door6.transform.Rotate(Vector3.up * f * -90);
                previousValue6 = ScrubF.val;
            }

        }		
    }
	
}